﻿
using HardwareShopContracts.BindingModels;
using HardwareShopContracts.SearchModels;
using HardwareShopContracts.StoragesContracts;
using HardwareShopContracts.ViewModels;
using HardwareShopDatabaseImplement.Implements.Storekeeper;
using HardwareShopDatabaseImplement.Implements.Worker;
using HardwareShopDatabaseImplement.Models.Storekeeper;
using HardwareShopDatabaseImplement.Models.Worker;
using System.Collections.Generic;
using System.ComponentModel;

namespace HardwareShopContracts.BusinessLogicsContracts
{
    public class WorkerReportLogic : IWorkerReportLogic
    {
        private readonly IComponentStorage _componentStorage;

        private readonly BuildStorage _buildStorage;

        private readonly IPurchaseStorage _purchaseStorage;

        private readonly IGoodStorage _goodStorage;

        private readonly ICommentStorage _commentStorage;

        public WorkerReportLogic(IComponentStorage componentStorage, BuildStorage buildStorage, IPurchaseStorage purchaseStorage, IGoodStorage goodStorage, ICommentStorage commentStorage)
        {
            _componentStorage = componentStorage;
            _buildStorage = buildStorage;
            _purchaseStorage = purchaseStorage;
            _goodStorage = goodStorage;
            _commentStorage = commentStorage;
        }

        /// <summary>
        /// Получение списка компонент с указанием, в каких покупках используются
        /// </summary>
        /// <returns></returns>
        public List<ReportPurchaseComponentViewModel> GetPurchaseComponent(List<PurchaseViewModel> purchaseList)
        {
            var list = new List<ReportPurchaseComponentViewModel>();

            foreach (var purchase in purchaseList)
            {
                var record = new ReportPurchaseComponentViewModel
                {
                    Id = purchase.Id,
                    Builds = new List<(string Build, int count, List<(string Component, int count)>)>(),
                    Goods = new List<(string Good, int count, List<(string Component, int count)>)>(),
                    TotalCount = 0,
                    TotalCost = 0
                };
                foreach (var good in purchase.PurchaseGoods)
                {
                    List<(string Component, int count)> componentList = new List<(string Component, int count)>();
                    int goodTotalCount = 0;
                    foreach (var component in good.Value.Item1.GoodsComponents)
                    {
                        componentList.Add(new(component.Value.Item1.ComponentName, component.Value.Item2));
                        goodTotalCount += component.Value.Item2;
                    }
                    record.Goods.Add(new(good.Value.Item1.GoodName, good.Value.Item2, componentList));
                    record.TotalCount += goodTotalCount * good.Value.Item2;
                }
                foreach (var build in purchase.PurchaseBuilds)
                {
                    List<(string Component, int count)> componentList = new List<(string Component, int count)>();
                    int buildTotalCount = 0;
                    foreach (var component in build.Value.Item1.BuildComponents)
                    {
                        componentList.Add(new(component.Value.Item1.ComponentName, component.Value.Item2));
                        buildTotalCount += component.Value.Item2;
                    }
                    record.Builds.Add(new(build.Value.Item1.BuildName, build.Value.Item2, componentList));
                    record.TotalCount += buildTotalCount * build.Value.Item2;
                }
                list.Add(record);
            }
            return list;
        }

        /// <summary>
        /// Получение списка покупок за определенный период
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ReportPurchaseViewModel> GetPurchase(ReportBindingModel model)
        {
            var list = new List<ReportPurchaseViewModel>();
            var purchases = _purchaseStorage.GetFilteredList(new PurchaseSearchModel { DateFrom = model.DateFrom, DateTo = model.DateTo });

            foreach (var purchase in purchases)
            {
                var record = new ReportPurchaseViewModel
                {
                    Id = purchase.Id,
                    Builds = new List<(string Build, int count, List<string>, List<(string Component, int count)>)>(),
                };
                foreach (var build in purchase.PurchaseBuilds)
                {
                    List<string> commentList = new List<string>();
                    foreach (var comment in build.Value.Item1.BuildComments)
                    {
                        commentList.Add(new(comment.Value.Text));
                    }
                    List<(string Component, int count)> componentList = new List<(string Component, int count)>();
                    foreach (var component in build.Value.Item1.BuildComponents)
                    {
                        componentList.Add(new(component.Value.Item1.ComponentName, component.Value.Item2));
                    }
                    record.Builds.Add(new (build.Value.Item1.BuildName, build.Value.Item2, commentList, componentList));
                }
            }
            return list;
        }


        /// <summary>
        /// Сохранение компонент с указаеним покупок в файл-Word
        /// </summary>
        /// <param name="model"></param>
        public void SaveComponentsToWordFile(ReportBindingModel model)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Сохранение компонент с указаеним покупок в файл-Excel
        /// </summary>
        /// <param name="model"></param>
        public void SaveDishComponentToExcelFile(ReportBindingModel model)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Сохранение отчёта по покупкам в файл-Pdf
        /// </summary>
        /// <param name="model"></param>
        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            throw new NotImplementedException();
        }
    }
}